@@ -4,7 +4,7 @@ Test nml.py functions/methods.
 
 File: neuroml/test/test_nml.py
 
-Copyright 2021 NeuroML contributors
+Copyright 2023 NeuroML contributors
 """
 
 try:
@@ -12,7 +12,7 @@ try:
 except ImportError:
     import unittest
 
-from neuroml.utils import component_factory
+from neuroml.utils import (component_factory, print_hierarchy)
 import neuroml
 
 
@@ -238,8 +238,11 @@ class TestNML(unittest.TestCase):
         """Test adding a segment."""
         new_cell = component_factory("Cell", id="test_cell")
         segment = new_cell.add_segment(
-            (0, 0, 0, 20), (20, 0, 0, 20), name="soma", group_id="soma_group",
-            seg_type="soma"
+            (0, 0, 0, 20),
+            (20, 0, 0, 20),
+            name="soma",
+            group_id="soma_group",
+            seg_type="soma",
         )
         self.assertIsInstance(segment, neuroml.Segment)
         self.assertEqual(segment.proximal.diameter, 20.0)
@@ -253,8 +256,7 @@ class TestNML(unittest.TestCase):
         """Test adding a segment but without a group."""
         new_cell = component_factory("Cell", id="test_cell")
         segment = new_cell.add_segment(
-            (0, 0, 0, 20), (20, 0, 0, 20), name="soma", group_id=None,
-            seg_type="soma"
+            (0, 0, 0, 20), (20, 0, 0, 20), name="soma", group_id=None, seg_type="soma"
         )
         self.assertIsInstance(segment, neuroml.Segment)
         self.assertEqual(segment.proximal.diameter, 20.0)
@@ -268,8 +270,11 @@ class TestNML(unittest.TestCase):
         """Test adding initial membrane potential."""
         new_cell = component_factory("Cell", id="test_cell")
         new_cell.add_segment(
-            (0, 0, 0, 20), (20, 0, 0, 20), name="soma", group_id="soma_group",
-            seg_type="soma"
+            (0, 0, 0, 20),
+            (20, 0, 0, 20),
+            name="soma",
+            group_id="soma_group",
+            seg_type="soma",
         )
         new_cell.set_init_memb_potential("-65mV")
 
@@ -279,8 +284,11 @@ class TestNML(unittest.TestCase):
         """Test adding spike threshold."""
         new_cell = component_factory("Cell", id="test_cell")
         new_cell.add_segment(
-            (0, 0, 0, 20), (20, 0, 0, 20), name="soma", group_id="soma_group",
-            seg_type="soma"
+            (0, 0, 0, 20),
+            (20, 0, 0, 20),
+            name="soma",
+            group_id="soma_group",
+            seg_type="soma",
         )
         new_cell.set_spike_thresh("40mV")
 
@@ -291,8 +299,11 @@ class TestNML(unittest.TestCase):
         """Units of membrane potential are wrong: should fail."""
         new_cell = component_factory("Cell", id="test_cell")
         new_cell.add_segment(
-            (0, 0, 0, 20), (20, 0, 0, 20), name="soma", group_id="soma_group",
-            seg_type="soma"
+            (0, 0, 0, 20),
+            (20, 0, 0, 20),
+            name="soma",
+            group_id="soma_group",
+            seg_type="soma",
         )
         # Make it invalid: wrong dimensions for membrane potential
         new_cell.set_init_memb_potential(new_cell, "-65 cm")
@@ -303,8 +314,11 @@ class TestNML(unittest.TestCase):
         """Test setting the resistivity."""
         new_cell = component_factory("Cell", id="test_cell")
         new_cell.add_segment(
-            (0, 0, 0, 20), (20, 0, 0, 20), name="soma", group_id="soma_group",
-            seg_type="soma"
+            (0, 0, 0, 20),
+            (20, 0, 0, 20),
+            name="soma",
+            group_id="soma_group",
+            seg_type="soma",
         )
         new_cell.set_resistivity("2000 ohm_cm")
         self.assertIsNone(new_cell.validate(True))
@@ -314,8 +328,11 @@ class TestNML(unittest.TestCase):
         """Test setting the resistivity."""
         new_cell = component_factory("Cell", id="test_cell")
         new_cell.add_segment(
-            (0, 0, 0, 20), (20, 0, 0, 20), name="soma", group_id="soma_group",
-            seg_type="soma"
+            (0, 0, 0, 20),
+            (20, 0, 0, 20),
+            name="soma",
+            group_id="soma_group",
+            seg_type="soma",
         )
         new_cell.set_resistivity("2000 kilO")
         self.assertIsNone(new_cell.validate(True))
@@ -324,8 +341,11 @@ class TestNML(unittest.TestCase):
         """Test setting the specific_capacitance."""
         new_cell = component_factory("Cell", id="test_cell")
         new_cell.add_segment(
-            (0, 0, 0, 20), (20, 0, 0, 20), name="soma", group_id="soma_group",
-            seg_type="soma"
+            (0, 0, 0, 20),
+            (20, 0, 0, 20),
+            name="soma",
+            group_id="soma_group",
+            seg_type="soma",
         )
         new_cell.set_specific_capacitance("1.0 uF_per_cm2")
         self.assertIsNone(new_cell.validate(True))
@@ -335,8 +355,11 @@ class TestNML(unittest.TestCase):
         """Test setting the specific_capacitance."""
         new_cell = component_factory("Cell", id="test_cell")
         new_cell.add_segment(
-            (0, 0, 0, 20), (20, 0, 0, 20), name="soma", group_id="soma_group",
-            seg_type="soma"
+            (0, 0, 0, 20),
+            (20, 0, 0, 20),
+            name="soma",
+            group_id="soma_group",
+            seg_type="soma",
         )
         new_cell.set_specific_capacitance("kilo")
         self.assertIsNone(new_cell.validate(True))
@@ -345,8 +368,11 @@ class TestNML(unittest.TestCase):
         """Test setting the channel_density."""
         new_cell = component_factory("Cell", id="test_cell")
         new_cell.add_segment(
-            (0, 0, 0, 20), (20, 0, 0, 20), name="soma", group_id="soma_group",
-            seg_type="soma"
+            (0, 0, 0, 20),
+            (20, 0, 0, 20),
+            name="soma",
+            group_id="soma_group",
+            seg_type="soma",
         )
         nml_doc = component_factory(
             "NeuroMLDocument", id="test_cell_with_channel_density_doc"
@@ -386,8 +412,11 @@ class TestNML(unittest.TestCase):
         """Test setting the channel_density."""
         new_cell = component_factory("Cell", id="test_cell")
         new_cell.add_segment(
-            (0, 0, 0, 20), (20, 0, 0, 20), name="soma", group_id="soma_group",
-            seg_type="soma"
+            (0, 0, 0, 20),
+            (20, 0, 0, 20),
+            name="soma",
+            group_id="soma_group",
+            seg_type="soma",
         )
         nml_doc = component_factory(
             "NeuroMLDocument", id="test_cell_with_channel_density_doc"
@@ -430,8 +459,11 @@ class TestNML(unittest.TestCase):
         """Test setting the channel_density."""
         new_cell = component_factory("Cell", id="test_cell")
         new_cell.add_segment(
-            (0, 0, 0, 20), (20, 0, 0, 20), name="soma", group_id="soma_group",
-            seg_type="soma"
+            (0, 0, 0, 20),
+            (20, 0, 0, 20),
+            name="soma",
+            group_id="soma_group",
+            seg_type="soma",
         )
         nml_doc = component_factory(
             "NeuroMLDocument", id="test_cell_with_channel_density_doc"
@@ -459,8 +491,11 @@ class TestNML(unittest.TestCase):
         """Test adding a new membrane property"""
         new_cell = component_factory("Cell", id="test_cell")
         new_cell.add_segment(
-            (0, 0, 0, 20), (20, 0, 0, 20), name="soma", group_id="soma_group",
-            seg_type="soma"
+            (0, 0, 0, 20),
+            (20, 0, 0, 20),
+            name="soma",
+            group_id="soma_group",
+            seg_type="soma",
         )
         new_cell.add_membrane_property("InitMembPotential", value="-65mV")
 
@@ -477,8 +512,11 @@ class TestNML(unittest.TestCase):
         """Test adding a new membrane property"""
         new_cell = component_factory("Cell", id="test_cell")
         new_cell.add_segment(
-            (0, 0, 0, 20), (20, 0, 0, 20), name="soma", group_id="soma_group",
-            seg_type="soma"
+            (0, 0, 0, 20),
+            (20, 0, 0, 20),
+            name="soma",
+            group_id="soma_group",
+            seg_type="soma",
         )
         new_cell.add_intracellular_property(
             "Resistivity",
@@ -507,7 +545,7 @@ class TestNML(unittest.TestCase):
             dist=[0.0, 10.0, 0.0, diam],
             name="Seg0_soma_0",
             group_id="soma_0",
-            seg_type="soma"
+            seg_type="soma",
         )
 
         self.assertIsInstance(soma_0, neuroml.Segment)
@@ -527,7 +565,7 @@ class TestNML(unittest.TestCase):
             dist=[0.0, 20.0, 0.0, diam],
             name="Seg0_soma_0",
             group_id="soma_0",
-            seg_type="soma"
+            seg_type="soma",
         )
         self.assertIsInstance(soma_0, neuroml.Segment)
 
@@ -537,7 +575,7 @@ class TestNML(unittest.TestCase):
             name="dend_0",
             group_id="dend_0",
             parent=soma_0,
-            seg_type="soma"
+            seg_type="soma",
         )
         self.assertIsInstance(dend_0, neuroml.Segment)
 
@@ -565,7 +603,7 @@ class TestNML(unittest.TestCase):
             dist=[0.0, 10.0, 0.0, diam],
             name="Seg0_soma_0",
             group_id="soma_0",
-            seg_type="soma"
+            seg_type="soma",
         )
 
         dend_group = cell.add_unbranched_segments(
@@ -578,7 +616,7 @@ class TestNML(unittest.TestCase):
             fraction_along=1.0,
             group_id="dend_group_0",
             use_convention=True,
-            seg_type="dendrite"
+            seg_type="dendrite",
         )
 
         self.assertIsInstance(dend_group, neuroml.SegmentGroup)
@@ -589,8 +627,7 @@ class TestNML(unittest.TestCase):
         cell.summary()
 
     def test_optimise_segment_group(self):
-        """Test `optimise_segment_group`
-        """
+        """Test `optimise_segment_group`"""
         cell = component_factory("Cell", id="simple_cell")  # type: neuroml.Cell
         cell.notes = "NeuroML cell created by CellBuilder"
 
@@ -601,23 +638,27 @@ class TestNML(unittest.TestCase):
             dist=[0.0, 10.0, 0.0, diam],
             name="Seg0_soma_0",
             group_id="soma_0",
-            seg_type="soma"
+            seg_type="soma",
         )
 
         # add the segment again to all group
-        cell.get_segment_group("all").members.append(cell.component_factory(neuroml.Member,
-                                                     segments=cell.get_segment_group("soma_0").members[0].segments))
+        cell.get_segment_group("all").members.append(
+            cell.component_factory(
+                neuroml.Member,
+                segments=cell.get_segment_group("soma_0").members[0].segments,
+            )
+        )
 
         # add group to all, explicitly
-        cell.get_segment_group("all").add(neuroml.Include,
-                                          segment_groups="soma_0",
-                                          force=True)
-        cell.get_segment_group("all").add(neuroml.Include,
-                                          segment_groups="soma_0",
-                                          force=True)
-        cell.get_segment_group("all").add(neuroml.Include,
-                                          segment_groups="soma_0",
-                                          force=True)
+        cell.get_segment_group("all").add(
+            neuroml.Include, segment_groups="soma_0", force=True
+        )
+        cell.get_segment_group("all").add(
+            neuroml.Include, segment_groups="soma_0", force=True
+        )
+        cell.get_segment_group("all").add(
+            neuroml.Include, segment_groups="soma_0", force=True
+        )
         self.assertEqual(4, len(cell.get_segment_group("all").includes))
         # should have only one included segment group
         # should have no segments, because the segment is included in the one
@@ -637,7 +678,7 @@ class TestNML(unittest.TestCase):
             dist=[0.0, 10.0, 0.0, diam],
             name="Seg0_soma_0",
             group_id="soma_0",
-            seg_type="soma"
+            seg_type="soma",
         )
 
         # create two unbranched segment group
@@ -651,7 +692,7 @@ class TestNML(unittest.TestCase):
             fraction_along=1.0,
             group_id="dend_group_0",
             use_convention=True,
-            seg_type="dendrite"
+            seg_type="dendrite",
         )
         dend_group_1 = cell.add_unbranched_segments(
             points=[
@@ -663,13 +704,15 @@ class TestNML(unittest.TestCase):
             fraction_along=1.0,
             group_id="dend_group_1",
             use_convention=True,
-            seg_type="dendrite"
+            seg_type="dendrite",
         )
         cell.optimise_segment_groups()
 
         self.assertIsNone(cell.validate(True))
         self.assertEqual(5, len(cell.morphology.segments))
-        self.assertEqual(7, len(cell.morphology.segment_groups))
+        # all, dend_group_1, dend_group_0, soma_0, and dendrite_group,
+        # soma_group
+        self.assertEqual(6, len(cell.morphology.segment_groups))
         print("initial")
         for g in cell.morphology.segment_groups:
             print(g)
@@ -690,3 +733,47 @@ class TestNML(unittest.TestCase):
         for g in cell.morphology.segment_groups:
             print(g)
             print([x.segments for x in g.members])
+
+    def test_morphinfo(self):
+        """Test the morphinfo method"""
+        # with component_factory
+        cell = component_factory("Cell", id="complex_cell")  # type: neuroml.Cell
+        cell.notes = "NeuroML cell created by CellBuilder"
+
+        # Add soma segment
+        diam = 30.0
+        soma_0 = cell.add_segment(
+            prox=[0.0, 0.0, 0.0, diam],
+            dist=[0.0, 20.0, 0.0, diam],
+            name="Seg0_soma_0",
+            group_id="soma_0",
+            seg_type="soma",
+        )
+        self.assertIsInstance(soma_0, neuroml.Segment)
+
+        dend_0 = cell.add_segment(
+            prox=[soma_0.distal.x, soma_0.distal.y, soma_0.distal.z, 5],
+            dist=[soma_0.distal.x, soma_0.distal.y + 50, soma_0.distal.z, 2],
+            name="dend_0",
+            group_id="dend_0",
+            parent=soma_0,
+            seg_type="soma",
+        )
+
+        cell.morphinfo(True)
+        cell.biophysinfo()
+
+    def test_class_hierarchy(self):
+        """Test the class hierarchy getter and printer
+        """
+        hier = neuroml.Cell.get_class_hierarchy()
+        self.assertIsNotNone(hier)
+        print()
+        print_hierarchy(hier)
+
+        hier = neuroml.Morphology.get_class_hierarchy()
+        self.assertIsNotNone(hier)
+        print()
+        print(hier)
+        print()
+        print_hierarchy(hier)
